@@ -1,4 +1,4 @@
-// $Id: chxid.c,v 1.7 2004/02/06 15:01:43 ensc Exp $    --*- c -*--
+// $Id: chxid.c,v 1.9 2004/02/19 22:29:20 ensc Exp $    --*- c -*--
 
 // Copyright (C) 2004 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
 //  
@@ -67,12 +67,9 @@ showVersion()
 }
 
 bool
-handleFile(char const *name, char const * display_name,
-	   struct stat const *exp_st)
+handleFile(char const *name, char const * display_name)
 {
-  int	rc = vc_set_iattr_compat(name, exp_st->st_dev, exp_st->st_ino,
-				 global_args->ctx, 0, VC_IATTR_XID,
-				 &exp_st->st_mode);
+  int	rc = vc_set_iattr(name, global_args->ctx, 0, VC_IATTR_XID);
   
   if (rc==-1) {
     perror(display_name);
@@ -97,4 +94,5 @@ fixupParams(struct Arguments UNUSED *args, int argc)
 
   args->ctx            = resolveCtx(args->ctx_str);
   args->do_display_dir = !args->do_recurse;
+  args->do_display_dot = true;
 }
