@@ -1,4 +1,4 @@
-// $Id: save_ctxinfo.c,v 1.10 2004/02/06 14:53:00 ensc Exp $    --*- c++ -*--
+// $Id: save_ctxinfo.c,v 1.13 2004/03/12 04:51:52 ensc Exp $    --*- c++ -*--
 
 // Copyright (C) 2003 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
 //  
@@ -31,10 +31,12 @@
 #include <fcntl.h>
 #include <stdio.h>
 #include <limits.h>
+#include <errno.h>
 
-#define ENSC_WRAPPERS_VSERVER 1
-#define ENSC_WRAPPERS_UNISTD  1
-#define ENSC_WRAPPERS_FCNTL   1
+#define ENSC_WRAPPERS_PREFIX	"save_ctxinfo: "
+#define ENSC_WRAPPERS_VSERVER	1
+#define ENSC_WRAPPERS_UNISTD	1
+#define ENSC_WRAPPERS_FCNTL	1
 #include <wrappers.h>
 
 int	wrapper_exit_code = 255;
@@ -73,21 +75,21 @@ int main(int argc, char *argv[])
     return 255;
   }
 
-  Ereadlink(runfile, dstfile, sizeof(dstfile));
+  EreadlinkD(runfile, dstfile, sizeof(dstfile));
   len  = utilvserver_fmt_uint(buf, ctx);
 
-  fd = Eopen(dstfile, O_EXCL|O_CREAT|O_WRONLY, 0644);
+  fd = EopenD(dstfile, O_EXCL|O_CREAT|O_WRONLY, 0644);
   if (write(fd, buf,     len) !=len  ||
       write(fd, "\n",    1)   !=1) {
-    perror("write()");
+    perror("save_ctxinfo: write()");
     return -1;
   }
   Eclose(fd);
 
   strcat(runfile, ".rev/");
-  strcat(runfile, buf);
+  strncat(runfile, buf, len);
   unlink(runfile);
-  Esymlink(argv[1], runfile);
+  EsymlinkD(argv[1], runfile);
 
   Eexecv(argv[2], argv+2);
 }
